note
	description: "[
         ]"
	author: "$Author$"
	file: "$File$"
	date: "$Date$"
	revision: "$Revision$"

class
	RL_JSON_UTF8_STRING
inherit
	RL_JSON_STRING
		redefine
			item,
			set_item,
			make
		end

create
	make,
	make_from_separate

feature {NONE} -- Initialization
	make (a_item: READABLE_STRING_GENERAL)
		do
			item := as_utf8(a_item)
			if not is_string_8_safe (item) then
				item := as_utf8(safe_string_8 (item.as_string_8))
			end
		end


feature -- Query
	item : RL_STRING_UTF8
	json_string : STRING_32
		do
			create Result.make_from_string ("%"")
			Result.append_string_general (as_string_32(item))
			Result.extend ('"')
		end

	html : STRING_32
		do
			create Result.make_from_string ("<p>")
			Result.append_string_general (as_string_32(item))
			Result.append ("</p>")
		end

feature -- Command
	set_item (a_value: READABLE_STRING_GENERAL)
		do
			item := as_utf8(a_value)
			if not is_string_8_safe (item) then
				item := as_utf8(safe_string_8(item.as_string_8))
			end
		end

	append_string (a_string: READABLE_STRING_GENERAL)
			-- Append `a_string' to `item'.
		local
			l_utfed_string: RL_STRING_UTF8
		do
			l_utfed_string := as_utf8 (a_string)
			if not is_string_8_safe (l_utfed_string) then
				item.append (as_utf8 (safe_string_8 (item.as_string_8)))
			end
		end

feature -- Conversion

	unescaped_string_8: STRING_8
			-- Unescaped ascii string from `item'.
			--| note: valid only if `item' does not encode any unicode character.
		local
			s: like item
		do
			s := item
			create Result.make (s.count)
			unescape_to_string_8 (Result)
		end


	representation: STRING_8
			-- String representation of `item' with escaped entities if any.
		do
			create Result.make (item.count + 2)
			Result.append_character ('%"')
			Result.append (item)
			Result.append_character ('%"')
		end

	unescape_to_string_8 (a_output: STRING_8)
			-- Unescape string `item' into `a_output'.
			--| note: valid only if `item' does not encode any unicode character.
		local
			s: like item
			i, n: INTEGER
			c: CHARACTER
		do
			s := item
			n := s.count
			from
				i := 1
			until
				i > n
			loop
				c := s [i]
				if c = '\' then
					if i < n then
						inspect s [i + 1]
						when '%"' then
							a_output.append_character ('%"')
							i := i + 2
						when '\' then
							a_output.append_character ('\')
							i := i + 2
						when '/' then
							a_output.append_character ('/')
							i := i + 2
						when 'b' then
							a_output.append_character ('%B')
							i := i + 2
						when 'f' then
							a_output.append_character ('%F')
							i := i + 2
						when 'n' then
							a_output.append_character ('%N')
							i := i + 2
						when 'r' then
							a_output.append_character ('%R')
							i := i + 2
						when 't' then
							a_output.append_character ('%T')
							i := i + 2
						when 'u' then
								--| Leave unicode \uXXXX unescaped
							a_output.append_character (c) -- '\'
							i := i + 1
						else
							a_output.append_character (c) -- '\'
							i := i + 1
						end
					else
						a_output.append_character (c) -- '\'
						i := i + 1
					end
				else
					a_output.append_character (c)
					i := i + 1
				end
			end
		end


feature {NONE} -- Implementation

 	is_hexadecimal (s: READABLE_STRING_8): BOOLEAN
 			-- Is `s' an hexadecimal value?
		local
			i: INTEGER
 		do
			from
				Result := True
				i := 1
			until
				i > s.count or not Result
			loop
				Result := s [i].is_hexa_digit
				i := i + 1
			end
 		end

	hexadecimal_to_natural_32 (s: READABLE_STRING_8): NATURAL_32
			-- Hexadecimal string `s' converted to NATURAL_32 value
		require
			s_not_void: s /= Void
			is_hexadecimal: is_hexadecimal (s)
		local
			i, nb: INTEGER
			char: CHARACTER
		do
			nb := s.count
			if nb >= 2 and then s.item (2) = 'x' then
				i := 3
			else
				i := 1
			end
			from
			until
				i > nb
			loop
				Result := Result * 16
				char := s.item (i)
				if char >= '0' and then char <= '9' then
					Result := Result + (char |-| '0').to_natural_32
				else
					Result := Result + (char.lower |-| 'a' + 10).to_natural_32
				end
				i := i + 1
			end
		end

	is_string_8_safe (a_string: RL_STRING_UTF8): BOOLEAN
			-- Is `a_string' safe as a JSON string?
		local
			l_character,l_null: CHARACTER
			l_is_escaped, l_is_unicode: BOOLEAN
			l_unicode_count: INTEGER
		do
			Result := True
			across a_string as ic_chars until Result = False loop
				l_character := ic_chars.item
				if l_character = l_null then
					Result := False
				elseif l_is_escaped then
					if l_is_unicode then
						inspect l_character
						when '0'..'9', 'a'..'f', 'A'..'F' then
							if l_unicode_count < 3 then
								l_unicode_count := l_unicode_count + 1
							else
								l_is_unicode := False
								l_is_escaped := False
							end
						else
							Result := False
						end
					elseif l_character = 'u' then
						l_is_unicode := True
						l_unicode_count := 0
					elseif (is_json_special_character (l_character) or is_json_special_control (l_character)) and then not (l_character = '%N') then
						l_is_escaped := False
					else
						Result := False
					end
				else
					if l_character = '%H' then -- '\' escape character.
						l_is_escaped := True
					elseif is_json_special_character (l_character) and l_character /= '/' then
						Result := False
					end
				end
			end
		end
	other_item (a_other : separate like Current) : like item
		do
			create Result.make_from_separate(a_other.item)
		end

end
