note
	description: "Summary description for {RL_AWARE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	RL_AWARE
feature
	c_ascending : INTEGER = 1
	c_descending : INTEGER = 2
	c_unsorted : INTEGER = 3
	c_hidden : STRING_32 = "RL_HIDDEN"
	update_counter_cell : CELL[INTEGER_64]
		once
			create Result.put(0)
		end
	update_counter : INTEGER_64
		do
			Result := update_counter_cell.item
		end
	increment_update_counter
		do
			update_counter_cell.put(update_counter_cell.item + 1)
		end
	update_table : HASH_TABLE[INTEGER_64,STRING_8]
		once
			create Result.make(1000)
		end

	token_redis_ids: separate RL_LIST [STRING_8]
		once ("process")
			create Result.make_empty
		end

	is_single_token_server: BOOLEAN
		once
			Result := arguments.index_of_word_option ("token") > 0
		end

	token: detachable STRING_32
		once
			if arguments.index_of_word_option ("token") > 0 then
				Result := arguments.argument (arguments.index_of_word_option ("token") + 1)
			end
		end

	user: detachable STRING_32
		once
			if arguments.index_of_word_option ("user") > 0 then
				Result := arguments.argument (arguments.index_of_word_option ("user") + 1)
			end
		end

	is_valid_uri (a_string: READABLE_STRING_GENERAL): BOOLEAN
			-- Is `a_string' a valid URI?
		local
			l_split: like {RL_STRING_UTF8}.split
		do
			-- Find an existing class that validates URIs or extend this for completeness.
			l_split := as_utf8(a_string).split (':')
			if l_split.count > 1 and then attached l_split.first as al_scheme and then attached l_split [2] as al_path then
				if not al_scheme.is_empty and then al_scheme [1].is_alpha and then not al_path.is_empty then
					Result := across al_scheme as ic_chars all (ic_chars.item.is_alpha_numeric or else ic_chars.item = '+' or else ic_chars.item = '.' or else ic_chars.item = '-') end
				end
			end
		end

	file_path (a_path : ITERABLE[ANY]) : STRING_32
		local
			l_path : FILE_NAME
			l_string : STRING_8
			l_delimited : RL_DELIMITED_TEXT_32
			l_items : RL_LIST[STRING_8]
			l_item: STRING_8
		do
			create l_path.make
			across a_path as ic_path loop
				if attached {STRING_GENERAL} ic_path.item as al_string then
					l_string :=  as_string_32(al_string)
					l_string.replace_substring_all ("/", "\")
					if al_string.has('\') then
						create l_delimited.make (l_string, '\')
						create l_items
						across l_delimited.to_array as ic_delimited loop
							if attached ic_delimited.item as al_array_item
							and then attached dereferenced(as_string_32(al_array_item)) as al_item then
								l_items.extend (al_item)
							end
						end
						across l_items as ic_items loop
							l_path.extend (ic_items.item)
						end
					elseif attached dereferenced(l_string) as al_value then
						l_path.extend (al_value)
					end
				end
			end
			Result := l_path.string
			Result.replace_substring_all ("/", "\")
			Result.replace_substring_all ("\\","\")
			Result.replace_substring_all ("\",operating_environment.directory_separator.out)
		end
	as_utf8 (a_string : separate READABLE_STRING_GENERAL) : RL_STRING_UTF8
		do
			if attached {RL_STRING_UTF8} local_string(a_string) as al_string then
				create Result.make_from_separate(al_string)
			elseif attached {STRING_8} local_string(a_string) as al_string then
				create Result.make_from_string(create {STRING_8}.make_from_separate (al_string))
			elseif attached {STRING_32} local_string(a_string) as al_string then
				create Result.make_from_string_32 (create {STRING_32}.make_from_separate (al_string))
			else
				create Result.make_from_string_32 (create {STRING_32}.make_from_separate (a_string.as_string_32))
			end
		end
	as_string_32 (a_string : separate READABLE_STRING_GENERAL) : STRING_32
		local
			l_result : separate STRING_32
		do
			if attached {STRING_32} a_string as al_string then
				l_result := al_string
			elseif attached {STRING_8} a_string as al_string then
				if utf_converter.is_valid_utf_8_string_8 (al_string) then
					l_result := encoder.decoded_string(al_string)
				else
					l_result := encoder.decoded_string (encoder.encoded_string (al_string))
				end
			else
				l_result := a_string.as_string_32
			end
			create Result.make_from_separate (l_result)
		end
	local_string (a_string : separate READABLE_STRING_GENERAL) : READABLE_STRING_GENERAL
		do
			if attached {READABLE_STRING_GENERAL} a_string as al_string then
				Result := al_string
			else
				if attached {separate RL_STRING_UTF8} a_string as al_string then
					create {RL_STRING_UTF8} Result.make_from_separate(al_string)
				elseif attached {separate STRING_8} a_string as al_string then
					create {STRING_8} Result.make_from_separate(al_string)
				else
					create {STRING_32} Result.make_from_separate(a_string)
				end
			end
		end

	local_string_8 (a_string : separate STRING_8) : STRING_8
		do
			if attached {STRING_8} a_string as al_key then
				Result := al_key
			else
				create Result.make_from_separate(a_string)
			end
		end

	utf_converter : UTF_CONVERTER
		once
			create Result
		end
	encoder : UTF8_ENCODER
		once
			create Result
		end
	environment : EXECUTION_ENVIRONMENT
		once
			create Result
		end
	platform : PLATFORM
		once
			create Result
		end
	unzip_command : STRING_8
		once
			Result := ""
			if platform.is_mac then
				Result := "/usr/bin/unzip"
			elseif platform.is_unix then
				Result := "/usr/bin/unzip"
			end
		end
	shasum_command : STRING_8
		once
			Result := "/usr/bin/shasum"
		end
	rm_command : STRING_8
		once
			Result := "/bin/rm"
		end
	dereferenced (a_value : STRING_32) : detachable STRING_32
		do
			if a_value.starts_with ("$")
			and then attached environment.item(a_value.substring (2, a_value.count)) as al_value then
				Result := al_value
			else
				Result := a_value
			end
		end
	parse_timer : RL_PROCESS_TIMER
		once
			create Result
			Result.interrupt
		end
	node_position_template : TUPLE[x,y,z : DOUBLE]
		do
			check false then end
		end
--	exceptions : EXCEPTIONS
--		once
--			create Result
--		end
	run_python(a_script : STRING_8;a_arguments : ARRAY[STRING_8];a_action : PROCEDURE[TUPLE[STRING_8]])
		local
			l_factory : BASE_PROCESS_FACTORY
			l_process : BASE_PROCESS
			l_list : ARRAYED_LIST[STRING_8]
			l_file : RAW_FILE
			l_file_name : FILE_NAME
		do
			create l_file_name.make_temporary_name
			create l_file.make_open_write (l_file_name.out)
			l_file.put_string (a_script)
			l_file.close
			create l_factory
			create l_list.make_from_array(a_arguments)
			l_list.put_front (l_file_name.out)
			l_process := l_factory.process_launcher ("python3", l_list, Void)
--			l_process.redirect_output_to_agent (a_action)
			l_process.launch
			l_process.wait_for_exit
		end

	detachable_string_from_separate(a_string : detachable separate READABLE_STRING_8) : detachable STRING_8
		do
			if attached a_string as al_string then
				create Result.make_from_separate(al_string)
			end
		end

	string_from_separate(a_string :  separate READABLE_STRING_8) : STRING_8
		do
			create Result.make_from_separate(a_string)
		end


	detachable_string_32_from_separate(a_string : detachable separate READABLE_STRING_GENERAL) : detachable STRING_32
		do
			if attached a_string as al_string then
				create Result.make_from_separate(al_string)
			end
		end

	string_32_from_separate(a_string :  separate READABLE_STRING_GENERAL) : STRING_32
		do
			create Result.make_from_separate(a_string)
		end
	parsed_html (a_string : READABLE_STRING_GENERAL) : STRING_32
		local
			l_parser : RL_HTML_PARSER
			l_first,l_last : INTEGER
			l_head,l_tail,l_string : STRING_32
		do
			Result := as_string_32(a_string)
			if Result /~ ""
			and then Result.substring_index ("</", 1) > 0 then
				l_string := Result.twin
				l_string.prune_all('%U')
				l_first := l_string.index_of ('<', 1)
				l_last := l_string.last_index_of ('>', l_string.count)
				if l_first > 0 and then l_last > l_first then
					create l_parser.make
					l_parser.parse_from_string_32 (l_string.substring (l_first, l_last))
					if not l_parser.error_occurred then
						if l_first > 1 then
							l_head := l_string.substring (1, l_first - 1)
						else
							l_head := ""
						end
						if l_last < l_string.count then
							l_tail := l_string.substring (l_last + 1, l_string.count)
						else
							l_tail := ""
						end
						Result := l_head + l_parser.content + l_tail
					else
						Result := l_string
					end
				end
			end
		end

	arguments : ARGUMENTS_32
		once
			create Result
		end
feature -- Logging

	connections_log_path: STRING_32
		local
			l_index: INTEGER
			l_path: PATH
			l_name: STRING_32
		once
			l_index := arguments.index_of_word_option ("connection_log")
			create l_path.make_from_string ({EXECUTION_ENVIRONMENT}.arguments.command_name)
			if attached l_path.entry as al_entry then
				l_path := al_entry
			end
			create l_name.make_from_string (l_path.name)
			if attached l_path.extension as l_extension then
				l_name.remove_tail (l_extension.count + 1)
			end
			if l_name.is_whitespace then
				l_name := "app"
			end
			if l_index > 0 and arguments.argument_count > l_index then
				Result := arguments.argument (l_index + 1)
			else
				Result := environment.current_working_path.out + "/" + l_name + "_connections.log"
			end
		end

	log_connection (a_peer: STRING_8)
		local
			l_log_file: RAW_FILE
			l_timestamp: RL_DATE
			l_name: STRING
		do
			create l_log_file.make (connections_log_path)
			create l_timestamp.make_now
			if l_log_file.exists then
				l_log_file.open_append
			else
				l_log_file.open_write
			end
			l_log_file.putstring (l_timestamp.timestamp + ": " + a_peer)
			l_log_file.put_new_line
			l_log_file.close
		end

	exceptions : EXCEPTIONS
		once
			create Result
		end

	dump_error_file
		local
			l_file: RAW_FILE
			l_date_time: RL_DATE
			l_directory: DIRECTORY
			l_log_file, l_oldest_file: RAW_FILE
		do
			if attached exceptions.exception_trace as al_trace then
				create l_date_time.make_now
				create l_directory.make (environment.current_working_path.out + "/logs")
				if l_directory.count > 9 then
					across l_directory.entries as ic_logs loop
						if ic_logs.cursor_index = 1 then
							create l_oldest_file.make_with_path (ic_logs.item)
						end
						if attached l_oldest_file then
							create l_log_file.make_with_path (ic_logs.item)
							if l_log_file.date < l_oldest_file.date then
								l_oldest_file := l_log_file
							end
						end
					end
					if attached l_oldest_file then
						l_oldest_file.delete
					end
				end
				create l_file.make_open_write (l_directory.path.out + "/trace_file_" + l_date_time.ccyymmdd.out + "_" + l_date_time.persecs.out + ".log")
				l_file.put_string (al_trace)
				l_file.close
			end
		end
end
