note
	description: "Summary description for {KD_JSON_ARRAY_NODE}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	RL_JSON_ARRAY
inherit
	RL_JSON_NODE[RL_LIST[RL_JSON_NODE[ANY]]]
		redefine
			tail_descendent,
			put_tail,
			json_ordered_string,
			truncated_to_integer,
			truncated_to_integer_64,
--			do_all,
			is_same,
			is_empty,
			display,
			debug_output,
			default_create
		end
create
	make_empty,
	make_from_separate,
	default_create

feature --Creation
	make_empty
		do
			create item
			is_open := True
		end
	default_create
		do
			make_empty
		end
feature -- Access
	json_string : STRING_32
		local
			l_items : RL_LIST[STRING_32]
		do
			create l_items
			Result := "[%N"
			across item as ic_children loop
				if not ic_children.item.is_empty then
					l_items.extend (ic_children.item.json_string)
				end
			end
			across l_items as ic_children loop
				Result.append(ic_children.item)
				if ic_children.cursor_index < l_items.count then
					Result.extend(',')
				end
				Result.extend ('%N')
			end
			Result.extend(']')
		end

	display : STRING_32
		do
			Result := ""
			if item.count = 1
			and then attached item.first as al_first then
				Result := al_first.display
			else
				across item as ic_items loop
					Result.append(ic_items.cursor_index.out + " : " + ic_items.item.display + "%N")
				end
			end
		end

	debug_output : STRING_32
		do
			Result := "["
			if attached item.first as al_first then
				Result.append(al_first.debug_output + "...")
			end
			Result.append("]")
		end

	html : STRING_32
		do
			Result := "<"
			Result := Result + tag + ">%N"
			across item as ic_children loop
				Result.extend ('%N')
				Result.append(ic_children.item.html)
			end
			Result.append("%N</" + tag + ">%N")
		end

	json_ordered_string: STRING_32
			-- <Precursor>
		local
			l_ordered_items: RL_LIST [STRING_32]
		do
			create l_ordered_items
			across item as ic_children loop
				if not ic_children.item.is_empty then
					l_ordered_items.extend (ic_children.item.json_ordered_string)
				end
			end
			l_ordered_items.sort_by (agent {STRING_32}.is_less)
			Result := "[%N"
			across l_ordered_items as ic_items loop
				Result.append (ic_items.item)
				if ic_items.cursor_index < l_ordered_items.count then
					Result.extend (',')
				end
				Result.extend ('%N')
			end
			Result.extend (']')
		end

	is_empty : BOOLEAN
		do
			Result := item.is_empty
		end

	truncated_to_integer : INTEGER do Result := unset_json_value end

	truncated_to_integer_64: INTEGER_64 do Result := unset_json_integer64_value end

	extend(a_node : RL_JSON_NODE[ANY])
		do
--			if not a_node.is_empty then
				item.extend (a_node)
--			end
		end

	insert (a_node: RL_JSON_NODE [ANY]; a_index: INTEGER)
		do
			if not a_node.is_empty then
				item.insert (a_node, a_index)
			end
		end

	append (a_other : RL_JSON_ARRAY)
		do
			across a_other.item as ic_items loop
				extend(ic_items.item)
			end
		end

	has (a_item: RL_JSON_NODE [ANY]): BOOLEAN
		local
			l_found: BOOLEAN
		do
			across item as ic_items until l_found loop
				l_found := ic_items.item.is_same (a_item)
			end
			Result := l_found
		end

	has_string_item (a_item: READABLE_STRING_GENERAL): BOOLEAN
		local
			l_found: BOOLEAN
		do
			across item as ic_items until l_found loop
				if attached {RL_JSON_NODE[READABLE_STRING_GENERAL]} ic_items.item as al_string then
					l_found := al_string.item.as_string_32 ~ a_item.as_string_32
				end
			end
			Result := l_found
		end

	prune_all_same (a_item: RL_JSON_NODE [READABLE_STRING_GENERAL])
			-- Remove all occurrences of nodes with the same item as `a_g` from Current, and collapse the list
			-- Absolutely room for improvement on the algorithm.
		local
			l_same_items : RL_LIST [RL_JSON_NODE [ANY]]
		do
			create l_same_items
			across item as ic_items loop
				if ic_items.item.is_same (a_item) then
					l_same_items.extend (ic_items.item)
				end
			end
			across l_same_items as ic_repeats loop
				item.prune_all (ic_repeats.item)
			end
		end

	is_same (a_other : detachable RL_JSON_ARRAY) : BOOLEAN
		do
			if attached a_other as al_other then
				Result := item.count = al_other.item.count
				if Result then
					Result :=
						across item as ic_item all
							ic_item.item.is_same(al_other.item[ic_item.cursor_index])
						end
				end
			end
		end

	extend_integer (a_value : INTEGER)
		do
			extend(create {RL_JSON_INTEGER}.make (a_value))
		end
	extend_string_32(a_value : STRING_32)
		do
			extend(create {RL_JSON_STRING_32}.make (a_value))
		end
	extend_string_8(a_value : STRING_8)
		do
			extend(create {RL_JSON_UTF8_STRING}.make (a_value))
		end
	extend_string_utf8(a_value : RL_STRING_UTF8)
		do
			extend(create {RL_JSON_UTF8_STRING}.make (a_value))
		end
feature {RL_JSON_NODE} --Implementation
	tail_descendent(a_key : ARRAY[ANY];a_start : INTEGER) : detachable RL_JSON_NODE[ANY]
		do
			if a_key.count >= a_start and then attached {INTEGER} a_key[a_start] as al_key then
				if a_start = a_key.count then
					Result := item[al_key]
				elseif attached item[al_key] as al_item then
					Result := al_item.tail_descendent(a_key,a_start + 1)
				end
			end
		end
	put_tail(a_value : RL_JSON_NODE[ANY];a_path : ARRAY[ANY];a_start : INTEGER)
		local
			l_structure : RL_JSON_STRUCTURE
			l_array : RL_JSON_ARRAY
			i : INTEGER
		do
			if attached {INTEGER} a_path[a_start] as al_key then
				if a_path.count > a_start then
					if attached item[al_key] as al_item then
						al_item.put_tail(a_value,a_path,a_start + 1)
					elseif attached {STRING_GENERAL} a_path[a_start+1] then
						create l_structure
						item.force(l_structure,al_key)
						l_structure.put_tail(a_value,a_path,a_start + 1)
					elseif attached {INTEGER} a_path[a_start+1] then
						create l_array
						item.force(l_array,al_key)
						l_array.put_tail(a_value,a_path,a_start + 1)
					end
				else
					from i := item.count + 1 until i > al_key - 1 loop
						item.force (create {RL_JSON_STRUCTURE}, i)
						i := i+1
					end
					item.force(a_value,al_key)
				end
			end
		end
	other_item (a_other : separate like Current) : like item
		local
			l_node : separate RL_JSON_NODE[separate ANY]
			l_item : RL_JSON_NODE[ANY]
		do
			create Result
			across a_other.item as ic_items loop
				l_node := ic_items.item
				Result.extend(separate_item(l_node))
			end
		end
end
