note
	description: "Abstraction for a terminal node in a JSON tree."
	author: "Mark Howard"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	RL_JSON_NODE[G]

inherit
	RL_HTTP_AWARE
		undefine
			out
		end

	RL_JAVASCRIPT_AWARE
		undefine
			out
		end

	RL_VERSIONED
		undefine
			debug_output,
			out
		end
feature {NONE} -- Initialization
	make (a_item : like item)
		do
			item := a_item
		end

feature -- Access

	debug_output : STRING_32
		do
			if attached {READABLE_STRING_GENERAL} item as al_item then
				Result := as_string_32(al_item)
			elseif attached item as al_item then
				Result := displayed(al_item)
			else
				Result := "<Void>"
			end
		end

	displayed(a_item : attached G) : STRING_8
		do
			create Result.make_from_separate(a_item.out)
		end

	tag : STRING_32
		do
			if attached tag_saved as al_saved then
				Result := al_saved
			else
				Result := "div"
				tag_saved := Result
			end
		end

	tag_saved : detachable like tag

	set_tag (a_tag : like tag)
		do
			tag_saved := a_tag
		end

	item : G

	is_empty : BOOLEAN
		do
		end

	is_open : BOOLEAN

	set_item (a_value: like item)
		do
			item := a_value
			update_version
		end

--	copy_item (a_other: like Current)
--		do
--			brian_leahy.to_be_reviewed ("Should we write our own feature for copying?  Would need redefines in array, hash, and string descendants.")
--			if attached a_other.item as al_other_item then
--				item := al_other_item.deep_twin
--			end
--		end

	set_is_open (a_value : BOOLEAN)
		do
			is_open := a_value
		end

	is_same (a_other : detachable RL_JSON_NODE[G]) : BOOLEAN
		do
			Result := attached a_other as al_other and then item ~ al_other.item
		end

	descendent (a_path : ARRAY[ANY]) : detachable RL_JSON_NODE[ANY]
		do
			Result := tail_descendent(a_path,1)
		end
	put (a_value : RL_JSON_NODE[ANY];a_path : ARRAY[ANY])
		do
			put_tail(a_value,a_path,1)
			update_version
		end

--	do_all (a_agent : PROCEDURE[RL_JSON_NODE[detachable ANY],detachable STRING_32])
--		do
--			a_agent(Current,Void)
--		end

	json_string : STRING_32
			-- String representation of `item' in JSON format.
		deferred
		end

	html : STRING_32
			-- String representation of `item' in HTML format.
		deferred
		end
	json_ordered_string: STRING_32
			-- A `json_string' with consistent ordering of elements.  Useful for text comparison.
		do
			Result := json_string
		end

	append_to_string (a_string: STRING_32)
			-- Append `json_string' to `a_string'.
		do
			a_string.append (json_string)
		end

	append_ordered_to_string (a_string: STRING_32)
			-- Append `json_ordered_string' to `a_string'.
		do
			a_string.append (json_ordered_string)
		end

	truncated_to_integer : INTEGER
		do
			if attached {INTEGER} item as al_integer then
				Result := al_integer
			elseif attached {REAL} item as al_real then
				Result := al_real.truncated_to_integer
			elseif attached {REAL_64} item as al_double then
				Result := al_double.truncated_to_integer
			elseif attached {STRING_GENERAL} item as al_string and then al_string.is_double then
				Result := al_string.to_double.truncated_to_integer
			end
		end

	truncated_to_integer_64: INTEGER_64
		do
			if attached {INTEGER} item as al_integer then
				Result := al_integer
			elseif attached {INTEGER_64} item as al_integer_64 then
				Result := al_integer_64
			elseif attached {REAL} item as al_real then
				Result := al_real.truncated_to_integer
			elseif attached {REAL_64} item as al_double then
				Result := al_double.truncated_to_integer
			elseif attached {STRING_GENERAL} item as al_string and then al_string.is_integer_64 then
				Result := al_string.to_integer_64
			elseif attached {STRING_GENERAL} item as al_string and then al_string.is_double then
				Result := al_string.to_double.truncated_to_integer
			end
		end

	out : STRING_8
		do
			if attached {READABLE_STRING_GENERAL} item as al_string then
				Result := as_utf8(al_string).as_string_8
			else
				Result := as_utf8(json_string).as_string_8
			end
		end

	display : STRING_32
		do
			if attached {READABLE_STRING_GENERAL} item as al_string then
				Result := as_string_32(al_string)
			else
				Result := json_string
			end
		end

	to_string_32 : STRING_32
		do
			if attached {STRING_32} item as al_string then
				Result := al_string
			else
				Result := as_string_32(json_string)
			end
		end

--	editable_text : RL_EDITABLE_TEXT
--		do
--			create Result.make_with_text(out)
--		end

--	model_label : RL_MODEL_LABEL
--		do
--			create Result.make_with_text(out)
--		end

	make_from_separate(a_other : separate like Current)
		do
			item := other_item(a_other)
		end
	other_item(a_other : separate like Current) : like item
		deferred
		end
feature {RL_JSON_NODE} --Implementation

	tail_descendent (a_path : ARRAY[ANY];a_index : INTEGER) : detachable RL_JSON_NODE[ANY]
		do
		end
	put_tail(a_value : ANY;a_path : ARRAY[ANY];a_start : INTEGER)
		do
		end
	separate_item(a_node : separate RL_JSON_NODE[separate ANY]) : RL_JSON_NODE[ANY]
		do
			if attached {RL_JSON_NODE[ANY]} a_node as al_node then
				Result := al_node
			elseif attached {separate RL_JSON_NULL} a_node as al_node then
				create {RL_JSON_NULL} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_LITERAL} a_node as al_node then
				create {RL_JSON_LITERAL} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_REAL_64} a_node as al_node then
				create {RL_JSON_REAL_64} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_BOOLEAN} a_node as al_node then
				create {RL_JSON_BOOLEAN} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_INTEGER_64} a_node as al_node then
				create {RL_JSON_INTEGER_64}Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_ARRAY} a_node as al_node then
				create {RL_JSON_ARRAY} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_STRING_32} a_node as al_node then
				create {RL_JSON_STRING_32} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_UTF8_STRING} a_node as al_node then
				create {RL_JSON_UTF8_STRING} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_INTEGER} a_node as al_node then
				create {RL_JSON_INTEGER} Result.make_from_separate(al_node)
			elseif attached {separate RL_JSON_STRUCTURE} a_node as al_node then
				create {RL_JSON_STRUCTURE} Result.make_from_separate(al_node)
			else
				check false then end
			end
		end
end
