note
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	RL_JSON_STRUCTURE
inherit
	RL_JSON_NODE[RL_STRING_32_TABLE[RL_JSON_NODE[ANY]]]
		rename
			make as kd_json_node_make
		redefine
			debug_output,
			tail_descendent,
			put_tail,
			json_ordered_string,
			truncated_to_integer,
			truncated_to_integer_64,
--			do_all,
			is_same,
			is_empty,
--			editable_text,
--			model_label,
			display,
			default_create
		end
create
	make,
	make_from_separate,
	default_create
feature
	make_empty
		do
			create item.make (10)
			is_open := True
		end
	default_create
		do
			make_empty
		end
	make (a_values : ARRAY[TUPLE[key : READABLE_STRING_GENERAL;value : detachable ANY]])
		do
			make_empty
			update(a_values)
		end
	debug_output : STRING_32
		do
			Result := json_string
		end
	string_value (a_key : READABLE_STRING_GENERAL) : detachable STRING_32
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_UTF8_STRING} item [l_key] as al_string then
				Result := as_string_32(al_string.unescaped_string_8)
			elseif attached {RL_JSON_STRING_32} item [l_key] as al_string_32 then
				Result := al_string_32.item
			end
		end

	string_8_value (a_key: READABLE_STRING_GENERAL): detachable STRING_8
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_UTF8_STRING} item [l_key] as al_string then
				Result := al_string.unescaped_string_8
			elseif attached {RL_JSON_STRING_32} item [l_key] as al_string_32 then
				create Result.make (al_string_32.item.count)
				across al_string_32.item as ic_string_32 loop
					Result.extend (ic_string_32.item.to_character_8)
				end
			end
		end

	string_utf8_value (a_key: READABLE_STRING_GENERAL): detachable RL_STRING_UTF8
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_STRING} item [l_key] as al_string then
				Result := as_utf8(al_string.item)
			end
		end

	integer_value (a_key : READABLE_STRING_GENERAL) :  INTEGER
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached item[l_key] as al_item then
				Result := al_item.truncated_to_integer
			else
				Result := unset_json_value
			end
		end

	integer_64_value (a_key: READABLE_STRING_GENERAL): INTEGER_64
		local
			l_key: STRING_32
		do
			l_key := as_string_32 (a_key)
			if attached item [l_key] as al_item then
				Result := al_item.truncated_to_integer_64
			else
				Result := unset_json_integer64_value
			end
		end

	real_64_value (a_key : READABLE_STRING_GENERAL) :  REAL_64
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached item [l_key] as al_item then
				if attached {RL_JSON_REAL_64} al_item as al_double then
					Result := al_double.item
				elseif attached {RL_JSON_INTEGER} al_item as al_integer then
					Result := al_integer.item
				elseif attached {RL_JSON_INTEGER_64} al_item as al_integer_64 then
					Result := al_integer_64.item
				elseif attached {RL_JSON_UTF8_STRING} al_item as al_string and then al_string.item.is_double then
					Result := al_string.item.to_double
				elseif attached {RL_JSON_STRING_32} al_item as al_string_32 and then al_string_32.item.is_double then
					Result := al_string_32.item.to_double
				end
			end
		end

	is_empty : BOOLEAN
		do
			Result := item.is_empty
		end

	truncated_to_integer : INTEGER do Result := unset_json_value end

	truncated_to_integer_64: INTEGER_64 do Result := unset_json_integer64_value end

	boolean_value (a_key: READABLE_STRING_GENERAL): BOOLEAN
			-- Boolean value at `a_key' if any.
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_BOOLEAN} item [l_key] as al_json_boolean then
				Result := al_json_boolean.item
			end
		end

	list_value (a_key : READABLE_STRING_GENERAL) : detachable RL_LIST [RL_JSON_NODE [ANY]]
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_ARRAY} item[l_key] as al_array then
				Result := al_array.item
			end
		end
	array_value (a_key : separate READABLE_STRING_GENERAL) : detachable RL_JSON_ARRAY
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_ARRAY} item[l_key] as al_array then
				Result := al_array
			end
		end
	structure_value (a_key : separate READABLE_STRING_GENERAL) : detachable RL_JSON_STRUCTURE
		local
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			if attached {RL_JSON_STRUCTURE} item[l_key] as al_structure then
				Result := al_structure
			end
		end
	put_string(a_string : READABLE_STRING_GENERAL; a_key : separate READABLE_STRING_GENERAL)
		local
			l_string : RL_JSON_UTF8_STRING
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			create l_string.make (as_string_32(a_string))
			item[l_key] := l_string
		end
	put_integer(a_integer : INTEGER;a_key : READABLE_STRING_GENERAL)
		local
			l_integer : RL_JSON_INTEGER
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			create l_integer.make (a_integer)
			item[l_key] := l_integer
		end
	put_integer_64(a_integer : INTEGER_64;a_key : READABLE_STRING_GENERAL)
		local
			l_integer : RL_JSON_INTEGER_64
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			create l_integer.make (a_integer)
			item[l_key] := l_integer
		end
	put_real_64(a_value : REAL_64;a_key : STRING_32)
		local
			l_value : RL_JSON_REAL_64
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			create l_value.make (a_value)
			item[l_key] := l_value
		end
	put_boolean(a_boolean : BOOLEAN;a_key : STRING_32)
		local
			l_boolean : RL_JSON_BOOLEAN
			l_key : STRING_32
		do
			l_key := as_string_32(a_key)
			create l_boolean.make (a_boolean)
			item[l_key] := l_boolean
		end
	copy_from (a_other : RL_JSON_STRUCTURE)
		do
			across a_other.item.table as ic_nodes loop
				item[ic_nodes.key]:= ic_nodes.item
			end
		end
	update (a_attributes : ARRAY[TUPLE[key : READABLE_STRING_GENERAL;value : detachable ANY]])
		local
			l_string : STRING_32
		do
			across a_attributes as ic_attributes loop
				if attached ic_attributes.item.value then
					l_string := as_string_32(ic_attributes.item.key)
					if attached {RL_JSON_NODE[ANY]} ic_attributes.item.value as al_value then
						item[l_string] := al_value
					elseif attached {STRING_8} ic_attributes.item.value as al_string then
						item[l_string] := create {RL_JSON_UTF8_STRING}.make(al_string)
					elseif attached {RL_STRING_UTF8} ic_attributes.item.value as al_string then
						item[l_string] := create {RL_JSON_UTF8_STRING}.make(al_string)
					elseif attached {STRING_32} ic_attributes.item.value as al_string then
						item[l_string] := create {RL_JSON_STRING_32}.make(al_string)
					elseif attached {INTEGER} ic_attributes.item.value as al_integer then
						item[l_string] := create {RL_JSON_INTEGER}.make(al_integer)
					elseif attached {INTEGER_64} ic_attributes.item.value as al_integer then
						item[l_string] := create {RL_JSON_INTEGER_64}.make(al_integer)
					elseif attached {REAL_64} ic_attributes.item.value as al_double then
						item[l_string] := create {RL_JSON_REAL_64}.make(al_double)
					elseif attached {REAL_32} ic_attributes.item.value as al_double then
						item[l_string] := create {RL_JSON_REAL_64}.make(al_double)
					elseif attached {BOOLEAN} ic_attributes.item.value as al_boolean then
						item[l_string] := create {RL_JSON_BOOLEAN}.make(al_boolean)
					elseif attached {RL_JAVASCRIPT_UNQUOTED} ic_attributes.item.value as al_unquoted then
						item[l_string] := create {RL_JSON_LITERAL}.make (al_unquoted.text)
					end
				end
			end
		end
	json_string_utf8 : RL_STRING_UTF8
		do
			Result := as_utf8(json_string)
		end
	json_string : STRING_32
		local
			l_child : RL_JSON_NODE[ANY]
			l_items : RL_LIST[TUPLE[key,value : STRING_32]]
			l_string: STRING_32
		do
			if item.is_empty then
				Result := "{}"
			else
				create l_items
				Result := "{%N"
				across item.table as ic_children loop
					l_child := ic_children.item
					l_string := l_child.json_string
					if l_string /~ "" then
						l_items.extend ([safe_string(ic_children.key),l_string])
					end
				end
				across l_items as ic_children loop
					Result.extend ('"')
					Result.append (ic_children.item.key)
					Result.append ("%" : ")
					Result.append (ic_children.item.value)
					if ic_children.cursor_index < l_items.count then
						Result.extend(',')
					end
					Result.extend ('%N')
				end
				Result.extend('}')
			end
		end

	merge (a_other : RL_JSON_STRUCTURE;a_overwrite : BOOLEAN)
		do
			across a_other.item.table as ic_other loop
				if a_overwrite or else not item.has(ic_other.key) then
					item[ic_other.key] := ic_other.item
				end
			end
		end

	html : STRING_32
		local
			l_child : RL_JSON_NODE[ANY]
			l_string: STRING_32
		do
			Result := "<"
			Result := Result + tag + " " --to preserve STRING_32
			across item.table as ic_children loop
				l_child := ic_children.item
				l_string := l_child.html
				Result.append (safe_string (ic_children.key))
				Result.append (l_string + " ")
				Result.extend ('%N')
			end
			Result.append("> </" + tag + ">%N")
		end

	json_ordered_string: STRING_32
			-- <Precursor>
		local
			l_ordered_keys: RL_LIST [STRING_32]
		do
			if item.is_empty then
				Result := "{}"
			else
				create l_ordered_keys.make_from_array (item.table.current_keys)
				l_ordered_keys.sort_by (agent {STRING_32}.is_less)
				Result := "{%N"
				across l_ordered_keys as ic_keys loop
					if attached item [ic_keys.item] as al_value
					and then al_value.json_string /~ "" then
						Result.extend ('"')
						Result.append (safe_string (ic_keys.item))
						Result.append ("%" : ")
						Result.append (al_value.json_ordered_string)
						if ic_keys.cursor_index < l_ordered_keys.count then
							Result.extend (',')
						end
						Result.extend ('%N')
					end
				end
				Result.extend ('}')
			end
		end

--	do_all(a_agent : PROCEDURE[RL_JSON_NODE[detachable ANY],detachable STRING_32])
--		do
--			across item as ic_structure loop
--				a_agent(ic_structure.item,ic_structure.key)
--				ic_structure.item.do_all (a_agent)
--			end
--		end

	is_same (a_other : detachable RL_JSON_STRUCTURE) : BOOLEAN
		do
			if attached a_other as al_other then
				Result := item.count = al_other.item.count
				if Result then
					Result := across item.table as ic_item all
						ic_item.item.is_same(al_other.item[ic_item.key])
					end
				end
			end
		end

--	editable_text : RL_EDITABLE_TEXT
--		local
--			l_text : STRING_32
--		do
--			if attached item["text"] as al_text then
--				l_text := al_text.to_string_32
--			else
--				l_text := to_string_32
--			end
--			create Result.make_with_text(l_text)
--		end

	display : STRING_32
		do
			Result := ""
			if item.count = 1
			and then attached item[("text").as_string_32] as al_text then
				Result.append (al_text.display)
			else
				across item.table as ic_items loop
					Result.append (ic_items.key + " : " + ic_items.item.display + "%N")
				end
			end
		end
--	model_label : RL_MODEL_LABEL
--		local
--			l_text : STRING_32
--		do
--			if attached  item["text"] as al_text then
--				l_text := al_text.to_string_32
--			else
--				l_text := to_string_32
--			end
--			create Result.make_with_text(l_text)
--		end

--	differences(a_other : RL_JSON_STRUCTURE) : RL_LIST[RL_LIST[ANY]]
--		local
--			l_differences : RL_LIST[ANY]
--		do
--			create Result
--			across item as ic_items loop
--				if attached a_other.item[ic_items.key] as al_other then
--					if not ic_items.item.is_same(al_other) then
--						if attached {RL_JSON_STRUCTURE} ic_items.item as al_structure then
--							across
--							l_differences.append (a_other: ITERABLE [G])
--						Result.extend(<<ic_items.key>>)
--					end
--				else
--					Result.extend(<<ic_items.key>>)
--				end
--			end
--		end

feature {RL_JSON_NODE} --Implementation
	tail_descendent(a_key : ARRAY[ANY];a_start : INTEGER) : detachable RL_JSON_NODE[ANY]
		local
			l_string : STRING_32
		do
			if a_key.count >= a_start and then attached {READABLE_STRING_GENERAL} a_key[a_start] as al_key then
				l_string := as_string_32(al_key)
				if a_start = a_key.count then
					Result := item[l_string]
				elseif attached item[l_string] as al_next then
					Result := al_next.tail_descendent(a_key,a_start + 1)
				end
			end
		end
	put_tail(a_value : RL_JSON_NODE[ANY];a_path : ARRAY[ANY];a_start : INTEGER)
		local
			l_structure : RL_JSON_STRUCTURE
			l_array : RL_JSON_ARRAY
			l_key : STRING_32
		do
			if attached {STRING_GENERAL} a_path[a_start] as al_key then
				l_key := as_string_32(al_key)
				if a_path.count > a_start then
					if attached item[l_key] as al_item then
						al_item.put_tail(a_value,a_path,a_start + 1)
					elseif attached {STRING_GENERAL} a_path[a_start+1] then
						create l_structure
						item[l_key] := l_structure
						l_structure.put_tail(a_value,a_path,a_start + 1)
					elseif attached {INTEGER} a_path[a_start+1] then
						create l_array
						item[l_key] := l_array
						l_array.put_tail(a_value,a_path,a_start + 1)
					end
				else
					item[l_key] := a_value
				end
			end
		end
	other_item (a_other : separate like Current) : like item
		local
			l_node : separate RL_JSON_NODE[separate ANY]
			l_item : RL_JSON_NODE[ANY]
		do
			create Result.make(a_other.item.count)
			across a_other.item.table as ic_items loop
				l_node := ic_items.item
				Result[ic_items.key] := separate_item(l_node)
			end
		end
end
