note
	description: "Extendible array with lower bound 1.  Put automatically extends, item will return default value if out of bounds, otherwise non void"
	author: "Mark Howard"
	date: "$Date$"
	revision: "$Revision$"

class
	RL_LIST[G]
inherit
--	INDEXABLE[G,INTEGER]
--		rename
--			at as attached_at
--		redefine
--			default_create,
--			index_set,
--			prune_all,
--			compare_objects
--		end
	ITERABLE[G]
		undefine
			default_create
		end
	DEBUG_OUTPUT
		redefine
			default_create
		end
create
	make_empty,
	make_from_array,
	make_from_iterable,
	make_from_linear,
	make_filled,
	default_create,
	make_conforming,
	make_projection
convert
	make_from_array({ARRAY[G]})
--	to_array : {ARRAY[G]}
feature {NONE} -- Initialization
	default_create
		do
			make_empty
		end
	make(a_count : INTEGER)
		do
			check false then end
			--create data.make(1,a_count)
		end
	make_filled(a_item : G;a_count : INTEGER)
		do
			create data.make_filled (a_item, 1, a_count)
		end
	make_from_array(a_array : ARRAY[G])
		do
			if a_array.count > 0 then
				make_filled(a_array[a_array.lower],a_array.count)
			else
				make_empty
			end
			across a_array as ic_data loop
				put (ic_data.item, ic_data.cursor_index)
			end
		end
	make_from_iterable(a_iterable : ITERABLE[G])
		local
			i : INTEGER
		do
			make_empty
			across a_iterable as ic_data loop
				extend (ic_data.item)
			end
		end
	make_from_linear(a_linear : LINEAR[G])
		do
			make_empty
			from a_linear.start until a_linear.off loop
				extend(a_linear.item)
				a_linear.forth
			end
		end
	make_conforming (a_iterable : ITERABLE[ANY])
		do
			make_empty
			across a_iterable as ic_iterable loop
				if attached {G} ic_iterable.item as al_item then
					extend(al_item)
				end
			end
		end
	make_projection (a_iterable : ITERABLE[ANY];a_agent : FUNCTION[TUPLE,detachable ANY])
		do
			make_empty
			across a_iterable as ic_iterable loop
				if attached {G} a_agent(ic_iterable.item) as al_item then
					extend(al_item)
				end
			end
		end
feature -- Access

	make_empty
		do
			create data.make_empty
		end
	new_cursor : RL_LIST_ITERATION_CURSOR[G]
		do
			create Result.make(Current)
			Result.forth
		end
	debug_output : READABLE_STRING_GENERAL
		do
			Result := count.out
		end
	force,put (a_item :  G;a_index : INTEGER)
		local
			l_count,i : INTEGER
			l_g : G
		do
			l_count := count
			data.force_and_fill (l_g, a_index)
			--force and fill puts a_item in every slot which is really undesirable
			data.put(a_item,a_index)
		end
	force_and_fill (a_item : G;a_index : INTEGER)
		do
			data.force_and_fill (a_item, a_index)
		end
	item alias "[]" (a_index : INTEGER) :  G assign put
		do
			check attached data[a_index] as al_item then
				Result := al_item
			end
		end
	at (a_index : INTEGER) : detachable G
		do
			if data.valid_index (a_index) then
				Result := data[a_index]
			end
		end
	is_empty : BOOLEAN
		do
			Result := data.is_empty
		end

--	is_same_content (a_other: like Current): BOOLEAN
--			-- Do Current and `a_other' have the same content, without regard to order?
--			--| NOTE: result is not reliable if either list has duplicate entries.
--		do
--			if count = a_other.count then
--				Result := across a_other as ic_items all
--					has (ic_items.item) and then a_other.has (item (ic_items.cursor_index))
--				end
--			end
--		end

	index_of(a_item : G) : INTEGER
		do
			across data as ic_current until Result > 0 loop
				if ic_current.item = a_item then
					Result := ic_current.cursor_index
				end
			end
		end

	sort_by(a_less: FUNCTION [attached G, attached G, BOOLEAN])
			-- Taken from algorithm provided in "Combinatorial Algorithms, Theory and Practice" by
			-- Edward M. Reingold, Jurg Nievergelt, and Narsingh Deo; Prentice-Hall Copyright 1977, page 289.
			-- isbn 0-13-152447-X.
		local
			stack_1: ARRAYED_STACK [INTEGER]
			stack_2: ARRAYED_STACK [INTEGER]
			f, i, j: INTEGER
			ell: INTEGER
			l_random: INTEGER
			l_temp_item: like item
			l_random_generator: RANDOM
		do
			create l_random_generator.make
			create stack_1.make (0)
			create stack_2.make (0)
			stack_1.put (0)
			stack_2.put (0)
			ell := count
			from
				f := 1
			until
				f >= ell
			loop
				l_random := next_item_in_range (l_random_generator, f, ell)
				if l_random /= f then
					l_temp_item := item (f)
					put (item (l_random), f)
					put (l_temp_item, l_random)
				end
				from
					i := f + 1
				until
					i > count or not detachable_less_than (item (i), item (f), a_less)
				loop
					i := i + 1
				end
				from
					j := ell
				until
					not detachable_less_than (item (f), item (j), a_less)
				loop
					j := j - 1
				end
				from
				until
					i >= j
				loop
					l_temp_item := item (i)
					put (item (j), i)
					put (l_temp_item, j)
					from
						i := i + 1
					until
						not detachable_less_than (item (i), item (f), a_less)
					loop
						i := i + 1
					end
					from
						j := j - 1
					until
						not detachable_less_than (item (f), item (j), a_less)
					loop
						j := j - 1
					end
				end
				if f /= j then
					l_temp_item := item (f)
					put (item (j), f)
					put (l_temp_item, j)
				end
				if ((j - 1) <= f) and (ell <= (j + 1)) then
					f := stack_1.item
					stack_1.remove
					ell := stack_2.item
					stack_2.remove
				elseif ((j - 1) <= f) and (ell > (j + 1)) then
					f := j + 1
				elseif ((j - 1) > f) and (ell <= (j + 1)) then
					ell := j - 1
				elseif ((j - 1) > f) and (ell > (j + 1)) then
					if (j - f) > (ell - j) then
						stack_1.put (f);
						stack_2.put (j - 1);
						f := j + 1
					else
						stack_1.put (j + 1);
						stack_2.put (ell);
						ell := j - 1
					end
				end
			end
		end
	next_item_in_range (random_generator: RANDOM; a_min: INTEGER; a_max: INTEGER): INTEGER
		local
			l_double: DOUBLE
		do
			random_generator.forth
			l_double := a_min + (a_max - a_min) * random_generator.double_item
			Result := l_double.rounded
		end
	index_set: INTEGER_INTERVAL
			-- Range of acceptable indexes
		do
			create Result.make (lower, upper)
		end
	valid_index(i : INTEGER) : BOOLEAN
		do
			Result := data.valid_index(i)
		end
	extend(a_g :  G)
		do
			data.force (a_g,count + 1)
		end
	extend_twin(a_g : detachable G)
		do
			if attached a_g as al_g then
				data.force (al_g.twin,count + 1)
			else
				data.force (a_g,count + 1)
			end
		end
	instance(a_g : G) : INTEGER
			--index of first instance, if any ... 0 otherwise
		do
			across data as ic_items until Result > 0 loop
				if ic_items.item ~ a_g then
					Result := ic_items.cursor_index
				end
			end
		end
	count : INTEGER
		do
			Result := data.count
		end

	lower : INTEGER
		do
			Result := 1
		end
	upper : INTEGER
		do
			Result := count
		end

	remove_at_index (a_index: INTEGER)
			-- Remove the item at `a_index', shifting remaining items left.
		require
			valid_index: valid_index (a_index)
		local
			l_count: INTEGER
		do
			l_count := count
			if a_index < l_count then
				swap (a_index, l_count)
			end
			restrict (l_count - 1)
		ensure
			one_less: count = old count - 1
			not_has_item_at_index: not has (old (item (a_index)))
		end

	occurrences (v: G): INTEGER
		do
			Result := data.occurrences (v)
		end

	extendible: BOOLEAN
			-- May new items be added?
		do
			Result := True
		end

--	prune (v: G)
--			-- Remove one occurrence of `v' if any.
--			-- (Reference or object equality,
--			-- based on `object_comparison'.)
--		local
--			l_removed : BOOLEAN
--			l_index : INTEGER
--		do
--			across data as ic_items until l_removed loop
--				if ic_items.item = v then
--					l_index := ic_items.cursor_index
--					l_removed := True
--				end
--			end
--			if l_removed then
--				remove_at_index (l_index)
--			end
--		end

	linear_representation: LINEAR [detachable G]
			-- Representation as a linear structure
		do
			Result := data.linear_representation
		end

	prunable: BOOLEAN
			-- May items be removed?
		do
			Result := True
		end

	prune_all(a_g : G)
			-- Remove all occurrences of `a_g' from Current, and collapse the list
		local
			l_data : like data
		do
			create l_data.make_empty
			across data as ic_data loop
				if ic_data.item /= a_g then
					l_data.force (ic_data.item, l_data.count + 1)
				end
			end
			data := l_data
		end

	duplicate  : RL_LIST[G]
		do
			create Result.make_empty
			across data as ic_current loop
				if attached ic_current.item as al_item then
					Result.extend(al_item)
				end
			end
		end
	reversed : RL_LIST[G]
		local
			i : INTEGER
		do
			create Result
			from i := count until i < 1 loop
				if attached item(i) as al_item then
					Result.extend(al_item)
				end
				i := i - 1
			end
		end
	generic : G
		do
			check false then end
		end
	has(a_g : detachable G) : BOOLEAN
		do
			Result := data.has (a_g)
		end
	first :  detachable G
		do
			if count > 0 then
				Result := item(1)
			end
		end
	last : detachable G
		do
			if count > 0 then
				Result := item(count)
			end
		end
	penultimate : detachable G
		do
			if count > 1 then
				Result := item(count-1)
			end
		end
	to_array : ARRAY[ G]
		do
			Result := collapsed
		end
	wipe_out
		do
			create data.make_empty
		end
	do_all (a_agent : PROCEDURE)
		do
			across data as i_items loop
				a_agent.call([i_items.item])
			end
		end
	restrict (a_count : INTEGER)
		do
			data.keep_head(a_count)
		end

	append (a_other : ITERABLE[ G])
		do
			across a_other as ic_items loop
				extend(ic_items.item)
			end
		end
	compare_objects
		do
			data.compare_objects
		end

	subarray (a_first,a_last : INTEGER) : like Current
		local
			i : INTEGER
		do
			Result := new_list
			from i := a_last until i > a_first loop
				Result.extend(item(i))
				i := i + 1
			end
		end

	subset (a_indices : ITERABLE[INTEGER]) : like Current
		do
			Result := new_list
			across a_indices as ic_indices loop
				if valid_index(ic_indices.item) then
					Result.extend(Current[ic_indices.item])
				end
			end
		end

	insert (a_value: G; a_index: INTEGER)
			-- Add `a_value' at `a_index', moving subsequent items
			-- to the right.
		require
			sensible_index : a_index > 0
		local
			l_old,l_new : G
			i : INTEGER
		do
			l_new := a_value
			from i := a_index until i > count loop
				l_old := item(i)
				put(l_new,i)
				l_new := l_old
				i := i + 1
			end
			force(l_new,count + 1)
		ensure
			new_count: count = old count + 1
			insertion_done: item (a_index) = a_value
		end

	inverse :  HASH_TABLE[INTEGER,INTEGER] --reverse map of attached elements if there are detached elements
		do
			if attached inverse_saved as al_inverse then
				Result := al_inverse
			else
				create Result.make(count)
				inverse_saved := Result
			end
		end
	binary_search (a_item : G;a_compare,a_equal : FUNCTION [G,G,BOOLEAN]) : TUPLE[lower_bound,upper_bound,found_index : INTEGER]
		local
			l_lower,l_upper,l_index: INTEGER
			l_found : BOOLEAN
			l_array : ARRAY[G]
			l_inverse : HASH_TABLE[INTEGER,INTEGER]
		do
			l_array := collapsed
			if a_compare.item ([a_item,l_array.item (1)])  then
				Result := [0,0,0]
			elseif a_compare.item ([l_array.item (count),a_item])  then
				Result := [count+1,count+1,0]
			elseif a_equal(a_item,l_array.item (count)) then
				Result := [count,count,count]
			elseif a_equal(a_item,l_array.item (1)) then
				Result := [1,1,1]
			else
				from
					l_lower := 1
					l_upper := count
				until
					l_lower >= l_upper - 1 or l_found
				loop
					l_index := (l_lower + l_upper) // 2
					l_found := a_equal(a_item,l_array.item (l_index))
					if not l_found then
						if a_compare.item ([l_array.item (l_index),a_item])  then
							l_lower := l_index
						else
							l_upper := l_index
						end
					else
						l_lower := l_index
						l_upper := l_index
					end
				end
				if not l_found then
					l_index := 0
				end
				if l_array.count < count then --accommodate void elements
					l_lower := inverse[l_lower]
					l_upper := inverse[l_upper]
					l_index := inverse[l_index]
				end
				Result := [l_lower,l_upper,l_index]
			end
		end
feature {NONE} --Implementation
	swap(i,j : INTEGER)
		local
			l_item : G
		do
			l_item := item(i)
			put(item(j),i)
			put(l_item,j)
		end

	data : ARRAY[detachable G]

	detachable_less_than (a_item1, a_item2: detachable G; a_less: FUNCTION [attached G, attached G, BOOLEAN]): BOOLEAN
			-- Is `a_item1' < `a_item2', according to `a_less' only if both items are attached.  A detached item is less than any attached item.
		do
			if attached a_item2 then
				if attached a_item1 then
					Result := a_less.item ([a_item1, a_item2])
				else
					Result := True
				end
			end
		end
feature {NONE} --Implementation

	collapsed : ARRAY[G]
		local
			i : INTEGER
		do
			create Result.make_empty
			inverse_saved := Void
			across data as ic_data loop
				if attached ic_data.item as al_item then
					Result.force(al_item,Result.count + 1)
				end
			end
			if Result.count < count then
				across data as ic_data loop
					if attached ic_data.item as al_item then
						i := i + 1
						inverse[i] := ic_data.cursor_index
					end
				end
			end
		end
	inverse_saved : detachable like inverse
	new_list : like Current
		do
			create Result
		end
end
